@@ -18,7 +18,6 @@ from typing import no_type_check, Dict, Type, Callable, get_origin, get_args, Se
 from bson import ObjectId
 
 from .error import SchemaError, DataError, PathError
-from .utils import parse_layout, iterate_layout
 
 
 # ----------------------------------------------------------------------------------------------------------------------
@@ -109,10 +108,11 @@ class SimpleEnum(object, metaclass=SimpleEnumMeta):
 class Format(SimpleEnum):
     """ Predefined available formats, which should be used to control ui or api generation.
 
-    Below values are the same with OAS 3.0
-    https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.0.3.md#data-types
-    https://swagger.io/docs/specification/data-models/data-types/
+    NOTE: Name of format can NOT end with interger, as #FormatInteger has special meaning in view layout, e.g, '#summary4' means with summary format and 4 span
     """
+    # Below values are the same with OAS 3.0
+    # - https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.0.3.md#data-types
+    # - https://swagger.io/docs/specification/data-models/data-types/
     DATETIME = 'datetime'  # Default format for datetime
     DATE = 'date'
     PASSWORD = 'password'
@@ -124,7 +124,6 @@ class Format(SimpleEnum):
     SELECT = 'select'  # Default format for SimpleEnum
     BUTTONGROUP = 'buttongroup'
     TAG = 'tag'  # Tag input
-    PASSWORD = 'password'
     TEXTAREA = 'textarea'
     RTE = 'rte'
     MARKDOWN = 'markdown'
@@ -140,7 +139,9 @@ class Format(SimpleEnum):
     LINK = 'link'  # Text input with an extenral link
     TIME = 'time'  # Time picker
     METRIC = 'metric'  # Statistic card for simple or object
+    RANGE = 'range'  # Range picker
     # Below values are used for inner model/dict or list of model/dict
+    SUMMARY = 'summary'  # Summary of a object, i.e, {}
     TAB = 'tab'  # Objects with tabs nav, i.e, [{}]
     TABLE = 'table'  # Objects in table, i.e, [{}]
     MODAL = 'modal'  # Objects with some fields in table and all fields in modal, i.e, [{}]
@@ -170,6 +171,17 @@ class Comparator(SimpleEnum):
     LIKE = 'like'  # Need to convert this to regex
 
 
+class Ownership(SimpleEnum):
+    """ Dependencies between two model, mainly used in many to one relation.
+
+    e.g,
+    task.project -> project.tasks, one project owns many tasks, means we can create task directly in a project detail page.
+    task.user -> user.tasks, one user has many tasks, but not owns them, means tasks should be created in other way and then you can assign them to different user.
+    """
+    HAVE = 'have'
+    OWN = 'own'
+
+
 # ----------------------------------------------------------------------------------------------------------------------
 # Constants
 #
@@ -331,6 +343,7 @@ class RelationField(ModelField):
         'back_field_title',
         'back_field_description',
         'back_field_unit',
+        'ownership',
         'is_back_field',
     )
 
@@ -339,16 +352,17 @@ class RelationField(ModelField):
                  save_field_name: str = Undefined, save_field_order: List[Tuple[str, int]] = [],
                  back_field_name: str = None, back_field_is_list: bool = False, back_field_order: List[Tuple[str, int]] = [],
                  back_field_format: Format = None, back_field_icon: str = None, back_field_title: str = None,
-                 back_field_description: str = None, back_field_unit: str = None, is_back_field: bool = False,
+                 back_field_description: str = None, back_field_unit: str = None,
+                 ownership: Ownership = None, is_back_field: bool = False,
                  **kwargs):
         """ Init method.
 
-        :param save_field_name: The name to use for saving current relation, if it is none, will use xxx_id/xxx_ids by default. The back field in related model have same value, used for search the original model
+        :param save_field_name: The name to use for saving current relation, if it is none, will use xxx_id/xxx_ids by default. The back field in related model have same value, used for searching the original model
         :param save_field_order: If the field is List, use this order when loading and saving
         :param back_field_name: The name to use for the relation from the related model back to this one, if it is none, do not create back field in related model
         :param back_field_is_list: If back field is list
         :param back_field_order: If back field is list, use this order when loading and saving
-        :param is_back_field: If its a back field defined in related object
+        :param is_back_field: If is a back field defined in related object
         """
         super().__init__(name, type_, **kwargs)
         # x12
@@ -362,6 +376,8 @@ class RelationField(ModelField):
         self.back_field_title = back_field_title
         self.back_field_description = back_field_description
         self.back_field_unit = back_field_unit
+        #
+        self.ownership = ownership
         self.is_back_field = is_back_field
 
 
@@ -508,9 +524,9 @@ class ModelMeta(ABCMeta):
                         editable=False,  # save field value will be updated by its source field, so no need to render in form
                         source_field_name=ann_name,  # Mark the source field
                     )
-                    # back_field_name should always has a meaningful value, if it is none, do not create back field in related object
+                    # back_field_name should always have a meaningful value, if it is none, do not create back field in related object
                     # e.g,
-                    # project.activities.user -> user, do not need to create back_field in user model as it is hard to search all the activities for user
+                    # project.activities.user -> user, no need to create back_field in user model as it is hard to search all the activities for user
                     # in such case, we need to create activity model, activity.project -> project.activities, activity.user -> user.activities
                     if field.back_field_name is None:
                         pass
@@ -646,6 +662,7 @@ class ModelMeta(ABCMeta):
                     editable=False,  # ditto
                     format_=f.back_field_format, icon=f.back_field_icon, title=f.back_field_title, description=f.back_field_description,
                     unit=f.back_field_unit,
+                    ownership=f.ownership,
                     is_back_field=True,  # Mark this field to be a back field created by a relation field
                 )
                 f_origin = get_origin(f.type)
@@ -679,28 +696,22 @@ class BaseModel(metaclass=ModelMeta):
     __icon__ = None
     __title__ = None
     __description__ = None
-    # TODO: Validate all fields in layout are defined
-    # Define fields can be show in query result table or card
-    __columns__ = []
-    # Define field groups, which can be included by index in layouts
-    # e.g,
-    # __groups__ = [
-    #   '''
-    #   name, status
-    #   phone
-    #   ''',
-    #   '''
-    #   password
-    #   ''',
-    # ]
-    # __read__ = '''
-    #   $, (0, 1)
-    # '''
-    __groups__ = []
-    # Define layouts to render read or form page
-    __layout__ = None
-    __read__ = None
-    __form__ = None
+    # Views for this model, should be a dict
+    # i.e,
+    # {
+    #   view_name: {                    # View name should be unique, in blueprint/view format, if blueprint is not specified, use default public by default
+    #     domains: ['www'],             # Domains that this view can be generated, should always be application's name
+    #     layout: '''action?param=1     # Layout of this view, we use this layout to generate corresponding view and template source code
+    #       field_string, field_select
+    #       field_image
+    #       field_object
+    #         field_string
+    #       field_textarea
+    #     ''',
+    #   },
+    #   ...
+    # }
+    __views__ = {}
 
     def __init__(self, *d: Dict[str, Any], **data: Any) -> None:
         """ Init.
@@ -906,14 +917,14 @@ class BaseModel(metaclass=ModelMeta):
         Note: When an attribute is accessed, __getattribute__ is invoked firstly
         and then if the attribute wasn't found, __getattr__ should be invoked.
         """
-        # Return depends data defined in a field, which can be function to return dynamic data
+        # Return _depends data defined in a field, which can be function to return dynamic data
         if name.endswith('_depends'):
             name = name.split('_depends')[0]
             field = self.__class__.__fields__.get(name)
             if field and field.depends:
                 depends = field.depends
                 if callable(depends):
-                    # check if depends function has arguments
+                    # check if _depends function has arguments
                     if depends.__code__.co_argcount == 0:
                         return depends()
                     else:
@@ -1126,7 +1137,7 @@ class BaseModel(metaclass=ModelMeta):
         return []
 
     @classmethod
-    def schema(cls):
+    def schema(cls, layouts={}):
         """ To json schema dict.
 
         NOTE:
@@ -1137,11 +1148,10 @@ class BaseModel(metaclass=ModelMeta):
 
         However, we still have some grammars, e.g,
           - add type date
-          - add enum_titles, py_type, layout, form, read, icon, editable to help code generation
+          - add enum_titles, py_type, icon, editable to help code generation
           - add format to array, so that we can gen a component for the whole array
           - add searchables to object, so that it can be used to generate search form
           - add sortables to object, so that it can be used to generate order drowpdown
-          - add columns to object, so that it can be used to generate columns for table
         """
 
         def _gen_schema(type_: Type, parents=[]):
@@ -1164,8 +1174,7 @@ class BaseModel(metaclass=ModelMeta):
                 #
                 properties = {}  # {field_name:field_schema}
                 requires, searchables, sortables = [], [], []
-                current_relations, current_form_relations = {}, {}  # {model_name:[field_name]}, use dict to keep the order
-                inner_relations, inner_form_relations = {}, {}  # {model_name:None}, ditto
+                relations = {}  # {model_name:[field_name]}, use dict to keep the order
                 for f_n, f_t in type_.__fields__.items():
                     field_schema = {}
                     f_origin = get_origin(f_t.type)
@@ -1180,9 +1189,8 @@ class BaseModel(metaclass=ModelMeta):
                         if f_type.__name__ in check_parents:
                             field_schema.update({
                                 'type': 'array',
-                                # https://json-schema.org/understanding-json-schema/structuring.html#recursion
-                                # Your program should replace the {$ref:} with reference object's schema
-                                'items': {'$ref': f'#{f_type.__name__}'},
+                                # Your program should use the reference object's schema to replace `items` value
+                                'items': {'ref': f_type.__name__},
                                 'py_type': f'List[{f_type.__name__}]',
                             })
                         else:
@@ -1198,9 +1206,9 @@ class BaseModel(metaclass=ModelMeta):
                         if f_type.__name__ in check_parents:
                             field_schema.update({
                                 'type': 'object',
-                                # Your program should replace the {$ref:} with properties of reference object's schema only
+                                # Your program should use the reference object's schema to replace `properties` under current schema
                                 # But should NOT overwrite the icon/title/description/... as these fields may use different values
-                                '$ref': f'#{f_type.__name__}',
+                                'ref': f_type.__name__,
                                 'py_type': f_type.__name__,
                             })
                         else:
@@ -1217,6 +1225,9 @@ class BaseModel(metaclass=ModelMeta):
                     # icon
                     if f_t.icon:
                         field_schema.update({'icon': f_t.icon})
+                    else:
+                        if inner_type and 'icon' in inner_type:
+                            field_schema.update({'icon': inner_type['icon']})
                     # title
                     field_schema.update({'title': f_t.title if f_t.title else f_n.upper()})
                     # description
@@ -1243,23 +1254,23 @@ class BaseModel(metaclass=ModelMeta):
                     # sortable
                     if f_t.sortable is True:
                         sortables.append(f_n)
-                    # depends, only update it to true, the you can access it by {field_name}_depends in page rendering or biz logic
+                    # depends, only update it to true, then you can access it by {field_name}_depends in page rendering or biz logic
                     if f_t.depends:
                         field_schema.update({'depends': True})
                     # relation
                     if isinstance(f_t, RelationField):
                         field_schema.update({'is_relation': True})
+                        #
+                        if f_t.ownership:
+                            field_schema.update({'ownership': f_t.ownership})
+                        #
                         field_schema.update({'is_out_relation': not f_t.is_back_field})
                         field_schema.update({'is_back_relation': f_t.is_back_field})
+                        #
                         field_schema.update({'save_field_name': f_t.save_field_name})
-                        # relations contains all the related model names, do not include any back relations
-                        if not f_t.is_back_field:
-                            current_relations.setdefault(f_type.__name__, [])
-                            current_relations[f_type.__name__] += [f_n]
-                            if inner_type and 'relations' in inner_type:
-                                inner_relations.update({k: None for k in inner_type['relations']})
-                                inner_form_relations.update({k: None for k in inner_type['form_relations']})
-
+                        #
+                        relations.setdefault(f_type.__name__, [])
+                        relations[f_type.__name__] += [f_n]
                     #
                     properties[f_n] = field_schema
                 #
@@ -1268,35 +1279,33 @@ class BaseModel(metaclass=ModelMeta):
                     'properties': properties,
                     'requires': requires,
                     'editable': True,
-                    'columns': type_.__columns__ if type_.__columns__ else requires,  # Using required fields as columns as the default
                     'py_type': type_.__name__,
                     'icon': type_.__icon__ if type_.__icon__ else None,
                     'title': type_.__title__ if type_.__title__ else type_.__name__.upper(),
                     'description': type_.__description__ if type_.__description__ else None,
+                    # searchables, [field__comparator|field]
+                    'searchables': [('{}__{}'.format(*s) if s[1] != Comparator.EQ else s[0]) for s in searchables],
+                    'searchable_fields': [s[0] for s in searchables],
+                    # sortables, [(field_name, 1/-1)]
+                    'sortables': sortables,
+                    'relations': [k for k in relations.keys() if k != type_.__name__],  # skip self
+                    # id_name & id_type, used for relation inputs
+                    'id_name': type_.__id_name__,
+                    'id_type': type_.__id_type__.__name__ if type_.__id_type__ else None,
                 }
-                # layout, if not defined, each field has one row
-                layout = parse_layout(type_.__layout__ if type_.__layout__ else '\n'.join(properties.keys()))[0]
-                obj['layout'] = layout
-                obj['read'] = parse_layout(type_.__read__)[0] if type_.__read__ else layout
-                obj['form'] = parse_layout(type_.__form__)[0] if type_.__form__ else layout
-                obj['groups'] = [parse_layout(g)[0] for g in type_.__groups__]
-                # each column in layout can be blank('')/hyphen(-)/summary($)/group(number)/field(string)/inner fields(has children) suffixed with query and span string
-                # read_fields/form_fields just return valid field names
-                obj['read_fields'] = list(iterate_layout(obj['read'], obj['groups']))
-                obj['form_fields'] = list(iterate_layout(obj['form'], obj['groups']))
-                # searchables, [field__comparator]
-                obj['searchables'] = [('{}__{}'.format(*s) if s[1] != Comparator.EQ else s[0]) for s in searchables]
-                obj['search_fields'] = [s[0] for s in searchables]
-                # sortables, [(field_name, 1/-1)]
-                obj['sortables'] = sortables
-                # relations recrusively
-                inner_relations.update(current_relations)
-                obj['relations'] = [k for k in inner_relations.keys() if k != type_.__name__]  # skip self because inner_relations may contains it
-                inner_form_relations.update({k: None for k, v in current_relations.items() if any((True for vv in v if vv in obj['form_fields']))})
-                obj['form_relations'] = [k for k in inner_form_relations.keys() if k != type_.__name__]
-                # id_name & id_type, used for relation inputs
-                obj['id_name'] = type_.__id_name__
-                obj['id_type'] = type_.__id_type__.__name__ if type_.__id_type__ else None
+                # Inject layouts into schema
+                if layouts:
+                    layout = layouts.get(type_.__name__, {})
+                    if not layout.get('columns'):
+                        layout['columns'] = obj['requires']
+                    if not layout.get('read'):
+                        # Each field is a row
+                        layout['read'] = [[{'name': f, 'params': {}}] for f in obj['properties'].keys()]
+                    if not layout.get('form'):
+                        layout['form'] = layout['read']
+                    #
+                    obj.update(layout)
+                #
                 return obj
             elif type_ is str:
                 return {'type': 'string', 'format': Format.TEXT, 'py_type': 'str'}
